package ch.zhaw.projekt2.turnierverwaltung;


import javafx.collections.FXCollections;
import javafx.collections.ObservableList;

import java.io.*;
import java.net.URI;
import java.util.logging.Logger;

/**
 * Class in Charge of Reading and Writing files
 */
public class FileIO {
    private File mainDir;
    private File saves;

    private static final Logger logger = Logger.getLogger(FileIO.class.getCanonicalName());

    /**
     * Constructor initiates the Directory and creates a save folder if not already existing.
     *
     * @param saveLocation the directory in which the save Files will be saved
     */
    public FileIO(String saveLocation) {
        this.mainDir = new File(saveLocation);
        if (!mainDir.exists()) {
            logger.fine("Creating main directory in given path" + saveLocation);
            mainDir.mkdir();
        } else {
            logger.finer("main directory already exists");
        }

        saves = new File(mainDir, "saves");
        if (!saves.exists()) {
            saves.mkdir();
            logger.fine("Creating save directory");
        } else {
            logger.finer("save directory already exists");
        }
    }


    /**
     * Returns a list with all save Files that are located inside the save folder.
     *
     * @return List with all containing save Files
     */
    public ObservableList<TournamentFile> getList() {
        logger.fine("Creating a List out of all Files in the save directory and returning it");
        ObservableList<TournamentFile> tournamentFiles = FXCollections.observableArrayList();
        for (File tournament : saves.listFiles()) {
            tournamentFiles.add(new TournamentFile(tournament.toURI()));
        }
        return tournamentFiles;
    }

    /**
     * Method to check if a tournament with the existing name already exists.
     * @param name that is being checked
     * @return true if the name exists already false if the name is unique
     */
    public boolean tournamentExists(String name) {
        logger.finer("checking for duplicate name in tournament List");
        for (TournamentFile file : getList()) {
            if (file.toString().toLowerCase().equals(name.toLowerCase())) {
                logger.fine(name + " is an already existing name in the list");
                return true;
            }
        }
        logger.fine(name + " is an unique name in the list");
        return false;
    }

    /**
     * Loads and returns a tournament from a given File which contains the serialiazed tournament.
     *
     * @param tournamentFile The tournament file where the data should be read from.
     * @return Tournament that is returned when succefully being read from the file
     * @throws ClassNotFoundException No definition for the class with the specified name could be found
     * @throws IOException            File not readable
     * @throws FileNotFoundException  File not found
     */
    public Tournament loadTournament(File tournamentFile) throws IOException, ClassNotFoundException, FileNotFoundException {
        if (tournamentFile == null) {
            logger.warning("Given tournament file is empty");
            throw new IllegalArgumentException("Tournament File is null");
        }
        Tournament tournament;
        logger.finer("Starting up Input Stream to read File");
        ObjectInputStream in = null;
        try {
            logger.fine("Setting up input file and reading it");
            FileInputStream fileInputStream = new FileInputStream(tournamentFile);
            in = new ObjectInputStream(fileInputStream);
            logger.finer("Starting to read tournament File");
            tournament = (Tournament) in.readObject();
        } catch (FileNotFoundException e) {
            logger.severe("Could not find tournament File" + e);
            throw e;
        } catch (IOException e) {
            logger.severe("Failed to read File" + tournamentFile.getName());
            throw new IOException("Error while reading File", e);
        } catch (ClassNotFoundException e) {
            logger.severe("No definition for the class with the specified name could be found" + e);
            throw new ClassNotFoundException("No definition for the class with the specified name could be found", e);
        } finally {
            if (in != null) {
                try {
                    logger.finer("Trying to close input stream");
                    in.close();
                } catch (IOException e) {
                    logger.severe("Failed to close input stream" + e);
                    throw new IOException("Error while closing input stream", e);
                }
            }
        }
        return tournament;
    }

    /**
     * Serializables and saves the receiving tournament file to a txt file.
     *
     * @param tournament the receiving tournament.
     * @throws IOException           File not readable
     * @throws FileNotFoundException File not found
     */
    public void saveTournament(Tournament tournament) throws IOException, FileNotFoundException {
        if (tournament == null) {
            logger.warning("Given tournament file is empty");
            throw new IllegalArgumentException("Null tournament received");
        }
        logger.fine("Creates the Object for the path of the save file");
        File newSave = new File(saves, tournament.getName() + ".txt");
        ObjectOutputStream out = null;

        try {
            logger.fine("creates or overwrites the new save FILE");
            newSave.createNewFile();
            out = new ObjectOutputStream(new FileOutputStream(newSave));
            out.writeObject(tournament);
            System.out.println("Save File " + tournament.getName() + ".txt being saved to " + saves.getAbsolutePath());

        } catch (FileNotFoundException e) {
            logger.severe("Could not find tournament File" + e);
            throw e;
        } catch (IOException e) {
            logger.severe("Failed to write File " + tournament.getName() + e);
            throw new IOException("Error while writing File", e);
        } finally {
            if (out != null) {
                try {
                    out.close();
                } catch (IOException e) {
                    logger.severe("Failed to close output stream" + e);
                    throw new IOException("Error while closing output stream", e);
                }
            }

        }

    }

    /**
     * Deletes the requested File in the directory
     *
     * @param tournamentFile the file that should be deleted
     * @throws FileNotFoundException    File could not be deleted
     * @throws IllegalArgumentException File is null
     */
    public void deleteTournament(File tournamentFile) throws FileNotFoundException {
        if (tournamentFile == null) {
            throw new IllegalArgumentException("Receiving file is null");
        }
        if (tournamentFile.delete()) {
            logger.fine("deleted requested File");
        } else {
            logger.warning("Failed to delete requested File");
            throw new FileNotFoundException("File deletion unsuccessful");
        }
    }

    /**
     * TournamentFile Class is in use to add missing functionality that is
     */
    public class TournamentFile extends File {

        /**
         * Only job the constructor got is to initialize it via its superclass. See java.io.File Documentation for more info.
         *
         * @param uri abstract pathname needed for its superclass to intialize the file accordingly.
         */
        public TournamentFile(URI uri) {
            super(uri);
        }

        /**
         * Method overrides toString to return the names of the tournaments without having .txt endings.
         *
         * @return String without a txt ending
         */
        @Override
        public String toString() {
            return getName().split("\\.")[0];
        }

    }

}
